package com.company.indexbugsoftdelete.entity
import io.jmix.core.entity.annotation.SystemLevel
import io.jmix.core.metamodel.annotation.DependsOnProperties
import io.jmix.core.metamodel.annotation.InstanceName
import io.jmix.core.metamodel.annotation.JmixEntity
import io.jmix.core.metamodel.annotation.JmixProperty
import javax.persistence.*
import javax.validation.constraints.NotEmpty
import javax.validation.constraints.NotNull

const val DEFAULT_SEARCH_KEY = ""

@JmixEntity
@MappedSuperclass
open class BaseMasterData : BaseEntitySoftDelete() {
    companion object {
        const val NAME = "NAME"
    }

    @NotEmpty
    @Column(name = "CODE", nullable = false)
    @NotNull
    var code: String? = null

    @NotNull
    @Column(name = "SEARCH_KEY", nullable = false)
    @SystemLevel
    var searchKey: String? = DEFAULT_SEARCH_KEY

    @NotEmpty
    @NotNull
    @Column(name = NAME)
    var name: String? = null

    @Column(name = "DESCRIPTION")
    var description: String? = null

    @Column(name = "SORT_NAME")
    var sortName: String? = null

    @JoinColumn(name = "BEST_CONFIDENCE_ORIGIN_ID")
    @ManyToOne(fetch = FetchType.LAZY)
    var bestConfidenceOrigin: DataOrigin? = null

    @get:JmixProperty
    @get:Transient
    @get:DependsOnProperties("searchKey")
    var searchKeyNotMandatory: String?
        get() = searchKey ?: DEFAULT_SEARCH_KEY
        set(value) {
            searchKey = value ?: DEFAULT_SEARCH_KEY
        }

    @DependsOnProperties("code", "name", "searchKey")
    override fun toString(): String = "$code-$name-$searchKey"

    fun fixNullSearchKey() {
        if (searchKey == null) searchKey = DEFAULT_SEARCH_KEY
    }

    @PrePersist
    open fun prePersist() = fixNullSearchKey()

    @PreUpdate
    open fun preUpdate() = fixNullSearchKey()

    @InstanceName
    @DependsOnProperties("code", "name")
    open fun getDisplayName(): String {
        return name ?: code!!
    }
}
