package com.company.indexbugsoftdelete.entity

import io.jmix.core.DeletePolicy
import io.jmix.core.Messages
import io.jmix.core.Metadata
import io.jmix.core.entity.annotation.OnDeleteInverse
import io.jmix.core.metamodel.annotation.InstanceName
import io.jmix.core.metamodel.annotation.JmixEntity
import io.jmix.core.metamodel.annotation.JmixProperty
import io.jmix.core.metamodel.model.MetaProperty
import java.util.*
import javax.annotation.PostConstruct
import javax.persistence.*
import javax.validation.constraints.NotNull

@JmixEntity
@Table(name = "COM_CUSTOM_FIELD", indexes = [
    Index(name = "IDX_COM_CUSTOM_FIELD_USER", columnList = "USER_ID"),
    Index(name = "IDX_COM_CUSTOM_FIELD", columnList = "ENTITY_CLASS, ENTITY_FIELD")
])
@Entity(name = "com_CustomField")
open class CustomField : BaseEntity() {
    @Column(name = "ENTITY_CLASS", nullable = false)
    @NotNull
    var entityClass: String? = null

    @Column(name = "ENTITY_FIELD", nullable = false)
    @NotNull
    var entityField: String? = null

    @Column(name = "MANDATORY")
    var mandatory: Boolean? = null

    @Column(name = "VISIBLE")
    var visible: Boolean? = null

    @InstanceName
    @Column(name = "CAPTION")
    var caption: String? = null

    @OnDeleteInverse(DeletePolicy.UNLINK)
    @JoinColumn(name = "USER_ID")
    @ManyToOne(fetch = FetchType.LAZY)
    var user: User? = null

    @JmixProperty
    @Transient
    var mandatoryInCode: Boolean? = null

    @JmixProperty
    @Transient
    var entityGroup: String? = null

    @JmixProperty
    @Transient
    var entityGroupI18n: String? = null

    @JmixProperty
    @Transient
    var entityClassI18n: String? = null

    @JmixProperty
    @Transient
    var entityFieldI18n: String? = null

    @PostConstruct
    open fun setComputedFields(messages: Messages, metadata: Metadata) {
        val field = entityField
        if (entityClass != null && field != null) {
            val metaClass = metadata.findClass(Class.forName(entityClass))
            val metaProperty = metaClass?.findProperty(field)
            setComputedFields(messages, metaProperty)
        }
    }
    open fun setComputedFields(messages: Messages, metaProperty: MetaProperty? = null) {
        val className = entityClass
        if (className != null) {
            entityGroup = className.substringBeforeLast(".")
            entityGroupI18n = messages.formatMessage(entityGroup!!.substringBeforeLast("."), "menu.addon")
            if (entityGroupI18n == "menu.addon") {
                entityGroupI18n = entityGroup?.substringBeforeLast(".")?.substringAfterLast(".")
                    ?.replaceFirstChar { if (it.isLowerCase()) it.titlecase(Locale.getDefault()) else it.toString() }
            }
            entityClassI18n = messages.formatMessage(entityGroup!!, className.substringAfterLast("."))
            if (entityField != null) {
                entityFieldI18n =
                    messages.formatMessage(metaProperty?.declaringClass?.packageName ?: entityGroup!!, "${metaProperty?.declaringClass?.simpleName ?: className.substringAfterLast(".")}.$entityField")
            }
        }
        if (metaProperty != null) {
            mandatoryInCode = null != metaProperty.annotatedElement.getDeclaredAnnotation(NotNull::class.java)
        }
    }
}
